@@ -1,21 +1,31 @@
 use crate::rule::{rule_by_name, Rule, Violation};
-use regex::Regex;
+use crate::utils::is_punctuation;
+use regex::{Regex, RegexBuilder};
 
 lazy_static! {
     pub static ref SUBJECT_WITH_MERGE_REMOTE_BRANCH: Regex = Regex::new(r"^Merge branch '.+' of .+ into .+").unwrap();
     static ref SUBJECT_STARTS_WITH_PREFIX: Regex = Regex::new(r"^([\w\(\)/!]+:)\s.*").unwrap();
-    static ref SUBJECT_STARTS_WITH_EMOJI: Regex = Regex::new(r"^\p{Emoji}").unwrap();
+    // Regex to match emoji, but not all emoji. Emoji using ASCII codepoints like the emojis for
+    // the numbers 0-9, and symbols like * and # are not included. Otherwise it would also catches
+    // plain numbers 0-9 and those symbols, even when they are not emoji.
+    // This regex matches all emoji but subtracts any object with ASCII codepoints.
+    // For more information, see:
+    // https://github.com/BurntSushi/ripgrep/discussions/1623#discussioncomment-28827
+    static ref SUBJECT_STARTS_WITH_EMOJI: Regex = Regex::new(r"^[\p{Emoji}--\p{Ascii}]").unwrap();
     static ref SUBJECT_WITH_TICKET: Regex = Regex::new(r"[A-Z]+-\d+").unwrap();
     // Match all GitHub and GitLab keywords
     static ref SUBJECT_WITH_FIX_TICKET: Regex =
         Regex::new(r"([fF]ix(es|ed|ing)?|[cC]los(e|es|ed|ing)|[rR]esolv(e|es|ed|ing)|[iI]mplement(s|ed|ing)?):? ([^\s]*[\w\-_/]+)?[#!]{1}\d+").unwrap();
+    static ref SUBJECT_WITH_CLICHE: Regex = {
+        let mut tempregex = RegexBuilder::new(r"^(fix(es|ed|ing)?|add(s|ed|ing)?|(updat|chang|remov|delet)(e|es|ed|ing))(\s+\w+)?$");
+        tempregex.case_insensitive(true);
+        tempregex.multi_line(false);
+        tempregex.build().unwrap()
+    };
+
     static ref URL_REGEX: Regex = Regex::new(r"https?://\w+").unwrap();
     static ref CODE_BLOCK_LINE_WITH_LANGUAGE: Regex = Regex::new(r"^\s*```\s*([\w]+)?$").unwrap();
     static ref CODE_BLOCK_LINE_END: Regex = Regex::new(r"^\s*```$").unwrap();
-    static ref OTHER_PUNCTUATION: Vec<char> = vec![
-        '…',
-        '⋯',
-    ];
     static ref MOOD_WORDS: Vec<&'static str> = vec![
         "fixed",
         "fixes",
@@ -93,14 +103,21 @@ lazy_static! {
 pub struct Commit {
     pub long_sha: Option<String>,
     pub short_sha: Option<String>,
+    pub email: Option<String>,
     pub subject: String,
     pub message: String,
     pub violations: Vec<Violation>,
+    pub ignored: bool,
     pub ignored_rules: Vec<Rule>,
 }
 
 impl Commit {
-    pub fn new(long_sha: Option<String>, subject: String, message: String) -> Self {
+    pub fn new(
+        long_sha: Option<String>,
+        email: Option<String>,
+        subject: String,
+        message: String,
+    ) -> Self {
         // Get first 7 characters of the commit SHA to get the short SHA.
         let short_sha = match &long_sha {
             Some(long) => match long.get(0..7) {
@@ -116,8 +133,10 @@ impl Commit {
         Self {
             long_sha,
             short_sha,
+            email,
             subject: subject.trim_end().to_string(),
             message,
+            ignored: false,
             ignored_rules,
             violations: Vec::<Violation>::new(),
         }
@@ -198,12 +217,12 @@ impl Commit {
     }
 
     fn validate_subject_line_length(&mut self) {
+        if self.rule_ignored(Rule::SubjectLength) {
+            return;
+        }
+
         let length = self.subject.chars().count();
         if length > 50 {
-            if self.rule_ignored(Rule::SubjectLength) {
-                return;
-            }
-
             self.add_violation(
                 Rule::SubjectLength,
                 format!(
@@ -213,10 +232,6 @@ impl Commit {
             )
         }
         if length < 5 {
-            if self.rule_ignored(Rule::SubjectLength) {
-                return;
-            }
-
             self.add_violation(
                 Rule::SubjectLength,
                 format!(
@@ -278,7 +293,7 @@ impl Commit {
                 if character.is_lowercase() {
                     self.add_violation(
                         Rule::SubjectCapitalization,
-                        "Start the commit subject a capital letter.".to_string(),
+                        "Start the commit subject with a capital letter.".to_string(),
                     )
                 }
             }
@@ -399,26 +414,11 @@ impl Commit {
 
         let subject = &self.subject.to_lowercase();
         let wip_commit = subject.starts_with("wip ") || subject == &"wip".to_string();
-        if wip_commit {
+        if wip_commit || SUBJECT_WITH_CLICHE.is_match(subject) {
             self.add_violation(
                 Rule::SubjectCliche,
                 "Reword the subject to describe the change in more detail.".to_string(),
             )
-        } else if subject == &"fix test".to_string() {
-            self.add_violation(
-                Rule::SubjectCliche,
-                "Reword the subject to explain which test was fixed.".to_string(),
-            )
-        } else if subject == &"fix bug".to_string() {
-            self.add_violation(
-                Rule::SubjectCliche,
-                "Reword the subject to explain what bug was fixed.".to_string(),
-            )
-        } else if subject == &"fix".to_string() {
-            self.add_violation(
-                Rule::SubjectCliche,
-                "Reword the subject to explain what was fixed.".to_string(),
-            )
         }
     }
 
@@ -513,10 +513,6 @@ impl Commit {
     }
 }
 
-fn is_punctuation(character: &char) -> bool {
-    character.is_ascii_punctuation() || OTHER_PUNCTUATION.contains(&character)
-}
-
 #[derive(PartialEq)]
 enum CodeBlockStyle {
     None,
@@ -529,7 +525,7 @@ mod tests {
     use super::{Commit, Rule, Violation, BUILD_TAGS, MOOD_WORDS};
 
     fn commit_with_sha(sha: Option<String>, subject: String, message: String) -> Commit {
-        Commit::new(sha, subject, message)
+        Commit::new(sha, Some("test@example.com".to_string()), subject, message)
     }
 
     fn commit(subject: String, message: String) -> Commit {
@@ -743,7 +739,17 @@ mod tests {
 
     #[test]
     fn test_validate_subject_punctuation() {
-        let subjects = vec!["Fix test", "あ commit"];
+        let subjects = vec![
+            "Fix test",
+            "あ commit",
+            "123 digits",
+            "0 digit",
+            // These should not be allowed, but won't match using the Emoji -- ASCII regex matcher.
+            // See the comment for SUBJECT_STARTS_WITH_EMOJI for more information.
+            "0️⃣ emoji",
+            "﹟emoji",
+            "＊emoji",
+        ];
         assert_commit_subjects_as_valid(subjects, &Rule::SubjectPunctuation);
 
         let invalid_subjects = vec![
@@ -920,26 +926,41 @@ mod tests {
     fn test_validate_subject_cliches() {
         let subjects = vec![
             "I am not a cliche",
+            "Fix user bug",
             "Fix test for some feature",
             "Fix bug for some feature",
+            "Fixes bug for some feature",
+            "Fixed bug for some feature",
+            "Fixing bug for some feature",
         ];
         assert_commit_subjects_as_valid(subjects, &Rule::SubjectCliche);
 
-        let invalid_subjects = vec![
-            "WIP something",
-            "wip something",
-            "Wip something",
-            "WIP",
-            "wip",
-            "Fix test",
-            "fix test",
-            "Fix bug",
-            "fix bug",
-            "Fix",
-            "fix",
-            "FIX",
+        let prefixes = vec![
+            "wip", "fix", "fixes", "fixed", "fixing", "add", "adds", "added", "adding", "update",
+            "updates", "updated", "updating", "change", "changes", "changed", "changing", "remove",
+            "removes", "removed", "removing", "delete", "deletes", "deleted", "deleting",
         ];
-        assert_commit_subjects_as_invalid(invalid_subjects, &Rule::SubjectCliche);
+        let mut invalid_subjects = vec![];
+        for word in prefixes.iter() {
+            let uppercase_word = word.to_uppercase();
+            let mut chars = word.chars();
+            let capitalized_word = match chars.next() {
+                None => panic!("Could not capitalize word: {}", word),
+                Some(letter) => letter.to_uppercase().collect::<String>() + chars.as_str(),
+            };
+
+            invalid_subjects.push(format!("{}", uppercase_word));
+            invalid_subjects.push(format!("{}", capitalized_word));
+            invalid_subjects.push(format!("{}", word));
+            invalid_subjects.push(format!("{} test", uppercase_word));
+            invalid_subjects.push(format!("{} issue", capitalized_word));
+            invalid_subjects.push(format!("{} bug", word));
+            invalid_subjects.push(format!("{} readme", word));
+            invalid_subjects.push(format!("{} something", word));
+        }
+        for subject in invalid_subjects {
+            assert_commit_subject_as_invalid(subject.as_str(), &Rule::SubjectCliche);
+        }
 
         let ignore_commit = validated_commit(
             "WIP".to_string(),
