@@ -1,6 +1,7 @@
 use regex::Regex;
-use std::process::Command;
 
+use crate::branch::Branch;
+use crate::command::run_command;
 use crate::commit::{Commit, SUBJECT_WITH_MERGE_REMOTE_BRANCH};
 
 const SCISSORS: &str = "------------------------ >8 ------------------------";
@@ -21,15 +22,23 @@ pub enum CleanupMode {
     Default,
 }
 
+pub fn fetch_and_parse_branch() -> Result<Branch, String> {
+    let name = run_command("git", &["rev-parse", "--abbrev-ref", "HEAD"])?
+        .trim()
+        .to_string();
+    let mut branch = Branch::new(name);
+    branch.validate();
+    Ok(branch)
+}
+
 pub fn fetch_and_parse_commits(selector: Option<String>) -> Result<Vec<Commit>, String> {
     let mut commits = Vec::<Commit>::new();
-    let mut command = Command::new("git");
-
     // Format definition per commit
     // Line 1: Commit SHA in long form
-    // Line 2 to second to last: Commit subject and message
+    // Line 2: Commit author email address
+    // Line 3 to second to last: Commit subject and message
     // Line last: Delimiter to tell commits apart
-    let format = "%H%n%B";
+    let format = "%H%n%ae%n%B";
     let mut args = vec![
         "log".to_string(),
         format!("--pretty={}{}", format, COMMIT_DELIMITER),
@@ -46,39 +55,30 @@ pub fn fetch_and_parse_commits(selector: Option<String>) -> Result<Vec<Commit>,
         None => args.push("HEAD~1..HEAD".to_string()),
     };
 
-    command.args(&args);
-    match command.output() {
-        Ok(raw_output) => {
-            let output = String::from_utf8_lossy(&raw_output.stdout);
-            let messages = output.split(COMMIT_DELIMITER);
-            for message in messages {
-                let trimmed_message = message.trim();
-                if !trimmed_message.is_empty() {
-                    match parse_commit(trimmed_message) {
-                        Some(commit) => commits.push(commit),
-                        None => debug!("Commit ignored: {:?}", message),
-                    }
-                }
+    let output = run_command("git", &args)?;
+    let messages = output.split(COMMIT_DELIMITER);
+    for message in messages {
+        let trimmed_message = message.trim();
+        if !trimmed_message.is_empty() {
+            match parse_commit(trimmed_message) {
+                Some(commit) => commits.push(commit),
+                None => debug!("Commit ignored: {:?}", message),
             }
         }
-        Err(e) => {
-            return Err(format!(
-                "Unable to fetch commits from Git: {:?}\n{}",
-                args, e
-            ))
-        }
-    };
+    }
     Ok(commits)
 }
 
 fn parse_commit(message: &str) -> Option<Commit> {
     let mut long_sha = None;
+    let mut email = None;
     let mut subject = None;
     let mut message_lines = vec![];
     for (index, line) in message.lines().enumerate() {
         match index {
             0 => long_sha = Some(line),
-            1 => subject = Some(line),
+            1 => email = Some(line.to_string()),
+            2 => subject = Some(line),
             _ => message_lines.push(line),
         }
     }
@@ -88,11 +88,12 @@ fn parse_commit(message: &str) -> Option<Commit> {
                 debug!("Commit subject not present in message: {:?}", message);
                 ""
             });
-            commit_or_none(
+            Some(commit_for(
                 Some(long_sha.to_string()),
+                email,
                 used_subject.to_string(),
                 message_lines,
-            )
+            ))
         }
         _ => {
             debug!("Commit ignored: SHA was not present: {}", message);
@@ -105,7 +106,7 @@ pub fn parse_commit_hook_format(
     message: &str,
     cleanup_mode: CleanupMode,
     comment_char: String,
-) -> Option<Commit> {
+) -> Commit {
     let mut subject = None;
     let mut message_lines = Vec::<&str>::new();
     let scissor_line = format!("{} {}", comment_char, SCISSORS);
@@ -141,22 +142,36 @@ pub fn parse_commit_hook_format(
         debug!("Commit subject not present in message: {:?}", message);
         ""
     });
-    commit_or_none(None, used_subject.to_string(), message_lines)
+    commit_for(None, None, used_subject.to_string(), message_lines)
 }
 
-fn commit_or_none(sha: Option<String>, subject: String, message: Vec<&str>) -> Option<Commit> {
-    let mut commit = Commit::new(sha, subject, message.join("\n"));
-    if !ignored(&commit) {
-        commit.validate();
-        Some(commit)
+fn commit_for(
+    sha: Option<String>,
+    email: Option<String>,
+    subject: String,
+    message: Vec<&str>,
+) -> Commit {
+    let mut commit = Commit::new(sha, email, subject, message.join("\n"));
+    if ignored(&commit) {
+        commit.ignored = true;
     } else {
-        None
+        commit.validate();
     }
+    commit
 }
 
 fn ignored(commit: &Commit) -> bool {
     let subject = &commit.subject;
     let message = &commit.message;
+    if let Some(email) = &commit.email {
+        if email.ends_with("[bot]@users.noreply.github.com") {
+            debug!(
+                "Ignoring commit because it is from a bot account: {}",
+                email
+            );
+            return true;
+        }
+    }
     if subject.starts_with("Merge tag ") {
         debug!(
             "Ignoring commit because it's a merge commit of a tag: {}",
@@ -200,10 +215,8 @@ fn ignored(commit: &Commit) -> bool {
 }
 
 pub fn cleanup_mode() -> CleanupMode {
-    let mut command = Command::new("git");
-    command.args(&["config", "commit.cleanup"]);
-    match command.output() {
-        Ok(raw_output) => match String::from_utf8_lossy(&raw_output.stdout).trim() {
+    match run_command("git", &["config", "commit.cleanup"]) {
+        Ok(stdout) => match stdout.trim() {
             "default" => CleanupMode::Default,
             "scissors" => CleanupMode::Scissors,
             "strip" => CleanupMode::Strip,
@@ -226,13 +239,9 @@ pub fn cleanup_mode() -> CleanupMode {
 }
 
 pub fn comment_char() -> String {
-    let mut command = Command::new("git");
-    command.args(&["config", "core.commentChar"]);
-    match command.output() {
-        Ok(raw_output) => {
-            let character = String::from_utf8_lossy(&raw_output.stdout)
-                .trim()
-                .to_string();
+    match run_command("git", &["config", "core.commentChar"]) {
+        Ok(stdout) => {
+            let character = stdout.trim().to_string();
             if character.is_empty() {
                 debug!("No Git core.commentChar config found. Using default `#` character.");
                 "#".to_string()
@@ -249,25 +258,46 @@ pub fn comment_char() -> String {
 
 #[cfg(test)]
 mod tests {
+    use super::Commit;
     use super::{parse_commit, parse_commit_hook_format, CleanupMode};
 
+    fn assert_commit_is_ignored(result: &Option<Commit>) {
+        match result {
+            Some(commit) => {
+                assert_eq!(commit.ignored, true);
+            }
+            None => panic!("Result is not a commit!"),
+        }
+    }
+
+    fn assert_commit_is_not_ignored(result: &Option<Commit>) {
+        match result {
+            Some(commit) => {
+                assert_eq!(commit.ignored, false);
+            }
+            None => panic!("Result is not a commit!"),
+        }
+    }
+
     #[test]
     fn test_parse_commit() {
         let result = parse_commit(
             "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\
+        test@example.com\n\
         This is a subject\n\
         \n\
         This is my multi line message.\n\
         Line 2.",
         );
 
-        assert!(result.is_some());
+        assert_commit_is_not_ignored(&result);
         let commit = result.unwrap();
         assert_eq!(
             commit.long_sha,
             Some("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa".to_string())
         );
         assert_eq!(commit.short_sha, Some("aaaaaaa".to_string()));
+        assert_eq!(commit.email, Some("test@example.com".to_string()));
         assert_eq!(commit.subject, "This is a subject");
         assert_eq!(commit.message, "\nThis is my multi line message.\nLine 2.");
         assert!(commit.violations.is_empty());
@@ -277,16 +307,18 @@ mod tests {
     fn test_parse_commit_with_errors() {
         let result = parse_commit(
             "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\
+        test@example.com\n\
         This is a subject",
         );
 
-        assert!(result.is_some());
+        assert_commit_is_not_ignored(&result);
         let commit = result.unwrap();
         assert_eq!(
             commit.long_sha,
             Some("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa".to_string())
         );
         assert_eq!(commit.short_sha, Some("aaaaaaa".to_string()));
+        assert_eq!(commit.email, Some("test@example.com".to_string()));
         assert_eq!(commit.subject, "This is a subject");
         assert_eq!(commit.message, "");
         assert!(!commit.violations.is_empty());
@@ -296,58 +328,74 @@ mod tests {
     fn test_parse_commit_empty() {
         let result = parse_commit("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n");
 
-        assert!(result.is_some());
+        assert_commit_is_not_ignored(&result);
         let commit = result.unwrap();
         assert_eq!(
             commit.long_sha,
             Some("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa".to_string())
         );
         assert_eq!(commit.short_sha, Some("aaaaaaa".to_string()));
+        assert_eq!(commit.email, None);
         assert_eq!(commit.subject, "");
         assert_eq!(commit.message, "");
         assert!(!commit.violations.is_empty());
     }
 
+    #[test]
+    fn test_parse_commit_ignore_bot_commit() {
+        let result = parse_commit(
+            "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\
+        12345678+bot-name[bot]@users.noreply.github.com\n\
+        Commit by bot without description",
+        );
+
+        assert_commit_is_ignored(&result);
+    }
+
     #[test]
     fn test_parse_commit_ignore_tag_merge_commit() {
         let result = parse_commit(
             "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\
+        test@example.com\n\
         Merge tag 'v1.2.3' into main",
         );
 
-        assert!(result.is_none());
+        assert_commit_is_ignored(&result);
     }
 
     #[test]
     fn test_parse_commit_ignore_merge_commit_pull_request() {
         let result = parse_commit(
             "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\
+        test@example.com\n\
         Merge pull request #123 from tombruijn/repo\n\
         \n\
         This is my multi line message.\n\
         Line 2.",
         );
 
-        assert!(result.is_none());
+        assert_commit_is_ignored(&result);
     }
 
     #[test]
     fn test_parse_commit_ignore_squash_merge_commit_pull_request() {
         let result = parse_commit(
             "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\
+        test@example.com\n\
         Fix some issue that's squashed (#123)\n\
         \n\
         This is my multi line message.\n\
         Line 2.",
         );
 
-        assert!(result.is_none());
+        assert_commit_is_ignored(&result);
     }
 
     #[test]
     fn test_parse_commit_ignore_merge_commits_merge_request() {
         let result = parse_commit(
             "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\
+        test@example.com\n\
         Merge branch 'branch' into main\n\
         \n\
         This is my multi line message.\n\
@@ -356,12 +404,13 @@ mod tests {
         See merge request gitlab-org/repo!123",
         );
 
-        assert!(result.is_none());
+        assert_commit_is_ignored(&result);
 
         // This is not a full reference, but a shorthand. GitLab merge commits
         // use the full org + repo + Merge Request ID reference.
         let result = parse_commit(
             "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\
+        test@example.com\n\
         Fix some issue\n\
         \n\
         This is my multi line message.\n\
@@ -370,10 +419,11 @@ mod tests {
         See merge request !123 for more info about the orignal fix",
         );
 
-        assert!(result.is_some());
+        assert_commit_is_not_ignored(&result);
 
         let result = parse_commit(
             "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\
+        test@example.com\n\
         Fix some issue\n\
         \n\
         This is my multi line message.\n\
@@ -382,64 +432,65 @@ mod tests {
         Also See merge request !123",
         );
 
-        assert!(result.is_some());
+        assert_commit_is_not_ignored(&result);
 
         let result = parse_commit(
             "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\
+        test@example.com\n\
         Fix some issue\n\
         \n\
         This is my multi line message.\n\
         Line 2. See merge request org/repo!123",
         );
 
-        assert!(result.is_some());
+        assert_commit_is_not_ignored(&result);
     }
 
     #[test]
     fn test_parse_commit_ignore_merge_commits_without_into() {
         let result = parse_commit(
             "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\
+        test@example.com\n\
         Merge branch 'branch'",
         );
 
-        assert!(result.is_none());
+        assert_commit_is_ignored(&result);
     }
 
     #[test]
     fn test_parse_commit_merge_remote_commits() {
         let result = parse_commit(
             "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\
+        test@example.com\n\
         Merge branch 'branch' of github.com/org/repo into branch",
         );
 
-        assert!(result.is_some());
+        assert_commit_is_not_ignored(&result);
     }
 
     #[test]
     fn test_parse_commit_hook_format() {
-        let result = parse_commit_hook_format(
+        let commit = parse_commit_hook_format(
             "This is a subject\n\nThis is a message.",
             CleanupMode::Default,
             "#".to_string(),
         );
 
-        assert!(result.is_some());
-        let commit = result.unwrap();
         assert_eq!(commit.long_sha, None);
         assert_eq!(commit.short_sha, None);
+        assert_eq!(commit.email, None);
         assert_eq!(commit.subject, "This is a subject");
         assert_eq!(commit.message, "\nThis is a message.");
     }
 
     #[test]
     fn test_parse_commit_hook_format_without_message() {
-        let result =
+        let commit =
             parse_commit_hook_format("This is a subject", CleanupMode::Default, "#".to_string());
 
-        assert!(result.is_some());
-        let commit = result.unwrap();
         assert_eq!(commit.long_sha, None);
         assert_eq!(commit.short_sha, None);
+        assert_eq!(commit.email, None);
         assert_eq!(commit.subject, "This is a subject");
         assert_eq!(commit.message, "");
     }
@@ -447,7 +498,7 @@ mod tests {
     // Same as Default
     #[test]
     fn test_parse_commit_hook_format_with_strip() {
-        let result = parse_commit_hook_format(
+        let commit = parse_commit_hook_format(
             "This is a subject\n\
             \n\
             This is the message body.  \n\
@@ -461,10 +512,9 @@ mod tests {
             "#".to_string(),
         );
 
-        assert!(result.is_some());
-        let commit = result.unwrap();
         assert_eq!(commit.long_sha, None);
         assert_eq!(commit.short_sha, None);
+        assert_eq!(commit.email, None);
         assert_eq!(commit.subject, "This is a subject");
         assert_eq!(
             commit.message,
@@ -474,7 +524,7 @@ mod tests {
 
     #[test]
     fn test_parse_commit_hook_format_with_strip_custom_comment_char() {
-        let result = parse_commit_hook_format(
+        let commit = parse_commit_hook_format(
             "This is a subject\n\
             \n\
             This is the message body.  \n\
@@ -488,10 +538,9 @@ mod tests {
             "-".to_string(),
         );
 
-        assert!(result.is_some());
-        let commit = result.unwrap();
         assert_eq!(commit.long_sha, None);
         assert_eq!(commit.short_sha, None);
+        assert_eq!(commit.email, None);
         assert_eq!(commit.subject, "This is a subject");
         assert_eq!(
             commit.message,
@@ -501,7 +550,7 @@ mod tests {
 
     #[test]
     fn test_parse_commit_hook_format_with_scissors() {
-        let result = parse_commit_hook_format(
+        let commit = parse_commit_hook_format(
             "This is a subject\n\
             \n\
             This is the message body.\n\
@@ -514,10 +563,9 @@ mod tests {
             "#".to_string(),
         );
 
-        assert!(result.is_some());
-        let commit = result.unwrap();
         assert_eq!(commit.long_sha, None);
         assert_eq!(commit.short_sha, None);
+        assert_eq!(commit.email, None);
         assert_eq!(commit.subject, "This is a subject");
         assert_eq!(
             commit.message,
@@ -527,7 +575,7 @@ mod tests {
 
     #[test]
     fn test_parse_commit_hook_format_with_verbatim() {
-        let result = parse_commit_hook_format(
+        let commit = parse_commit_hook_format(
             "This is a subject\n\
             \n\
             This is the message body.\n\
@@ -539,10 +587,9 @@ mod tests {
             "#".to_string(),
         );
 
-        assert!(result.is_some());
-        let commit = result.unwrap();
         assert_eq!(commit.long_sha, None);
         assert_eq!(commit.short_sha, None);
+        assert_eq!(commit.email, None);
         assert_eq!(commit.subject, "This is a subject");
         assert_eq!(
             commit.message,
@@ -556,7 +603,7 @@ mod tests {
 
     #[test]
     fn test_parse_commit_hook_format_with_whitespace() {
-        let result = parse_commit_hook_format(
+        let commit = parse_commit_hook_format(
             "This is a subject\n\
             \n\
             This is the message body.  \n\
@@ -570,10 +617,9 @@ mod tests {
             "#".to_string(),
         );
 
-        assert!(result.is_some());
-        let commit = result.unwrap();
         assert_eq!(commit.long_sha, None);
         assert_eq!(commit.short_sha, None);
+        assert_eq!(commit.email, None);
         assert_eq!(commit.subject, "This is a subject");
         assert_eq!(
             commit.message,
