@@ -12,19 +12,23 @@ use std::io::Read;
 use std::path::{Path, PathBuf};
 use structopt::StructOpt;
 
+mod branch;
+mod command;
 mod commit;
 mod git;
 mod logger;
 mod rule;
+mod utils;
 
+use branch::Branch;
 use commit::Commit;
-use git::{fetch_and_parse_commits, parse_commit_hook_format};
+use git::{fetch_and_parse_branch, fetch_and_parse_commits, parse_commit_hook_format};
 use logger::Logger;
 
 #[derive(StructOpt, Debug)]
 #[structopt(name = "lintje", verbatim_doc_comment)]
 /**
-Lint Git commits
+Lint Git commits and branch name.
 
 ## Usage examples
 
@@ -45,6 +49,9 @@ Lint Git commits
 
     lintje --hook-message-file=.git/COMMIT_EDITMSG
       Lints the given commit message file from the commit-msg hook.
+
+    lintje --no-branch
+      Disable branch name validation.
 */
 struct Lint {
     /// Prints debug information
@@ -55,6 +62,10 @@ struct Lint {
     #[structopt(long, parse(from_os_str))]
     hook_message_file: Option<PathBuf>,
 
+    /// Disable branch validation
+    #[structopt(long = "no-branch")]
+    no_branch_validation: bool,
+
     /// Lint commits by Git commit SHA or by a range of commits. When no <commit> is specified, it
     /// defaults to linting the latest commit.
     #[structopt(name = "commit (range)")]
@@ -64,31 +75,27 @@ struct Lint {
 fn main() {
     let args = Lint::from_args();
     init_logger(args.debug);
-    let result = match args.hook_message_file {
-        Some(hook_message_file) => lint_hook(&hook_message_file),
-        None => lint(args),
+    let commit_result = match args.hook_message_file {
+        Some(hook_message_file) => lint_commit_hook(&hook_message_file),
+        None => lint_commit(args.selection),
     };
-    match result {
-        Ok(_) => (),
-        Err(e) => {
-            error!("An error occurred: {}", e);
-            std::process::exit(2)
-        }
-    }
+    let branch_result = if args.no_branch_validation {
+        None
+    } else {
+        Some(lint_branch())
+    };
+    handle_lint_result(commit_result, branch_result, args.debug);
 }
 
-fn lint(options: Lint) -> Result<(), String> {
-    let commit_result = fetch_and_parse_commits(options.selection);
-    let commits = match commit_result {
-        Ok(commits) => commits,
-        Err(e) => return Err(e),
-    };
-
-    handle_lint_result(commits);
-    Ok(())
+fn lint_branch() -> Result<Branch, String> {
+    fetch_and_parse_branch()
 }
 
-fn lint_hook(filename: &Path) -> Result<(), String> {
+fn lint_commit(selection: Option<String>) -> Result<Vec<Commit>, String> {
+    fetch_and_parse_commits(selection)
+}
+
+fn lint_commit_hook(filename: &Path) -> Result<Vec<Commit>, String> {
     let commits = match File::open(filename) {
         Ok(mut file) => {
             let mut contents = String::new();
@@ -102,10 +109,9 @@ fn lint_hook(filename: &Path) -> Result<(), String> {
                     ));
                 }
             };
-            match parse_commit_hook_format(&contents, git::cleanup_mode(), git::comment_char()) {
-                Some(commit) => vec![commit],
-                None => vec![],
-            }
+            let commit =
+                parse_commit_hook_format(&contents, git::cleanup_mode(), git::comment_char());
+            vec![commit]
         }
         Err(e) => {
             return Err(format!(
@@ -115,36 +121,94 @@ fn lint_hook(filename: &Path) -> Result<(), String> {
             ));
         }
     };
-    handle_lint_result(commits);
-    Ok(())
+    Ok(commits)
 }
 
-fn handle_lint_result(commits: Vec<Commit>) {
-    debug!("Commits: {:?}", commits);
-    let commit_count = commits.len();
+fn handle_lint_result(
+    commit_result: Result<Vec<Commit>, String>,
+    branch_result: Option<Result<Branch, String>>,
+    debug: bool,
+) {
     let mut violation_count = 0;
-    for commit in commits {
-        if !commit.is_valid() {
-            match commit.short_sha {
-                Some(sha) => println!("{}: {}", sha, commit.subject),
-                None => println!("{}", commit.subject),
-            }
+    let mut commit_count = 0;
+    let mut ignored_commit_count = 0;
+    let mut branch_message = "";
 
-            for violation in commit.violations {
-                violation_count += 1;
-                println!("  {}: {}", violation.rule, violation.message);
+    if let Ok(ref commits) = commit_result {
+        debug!("Commits: {:?}", commits);
+        for commit in commits {
+            if commit.ignored {
+                ignored_commit_count += 1;
+                continue;
             }
+            commit_count += 1;
+            if !commit.is_valid() {
+                match &commit.short_sha {
+                    Some(sha) => println!("{}: {}", sha, commit.subject),
+                    None => println!("{}", commit.subject),
+                }
+
+                for violation in &commit.violations {
+                    violation_count += 1;
+                    println!("  {}: {}", violation.rule, violation.message);
+                }
+            }
+        }
+    }
+    let mut branch_error = None;
+    if let Some(result) = branch_result {
+        match result {
+            Ok(ref branch) => {
+                debug!("Branch: {:?}", branch);
+                branch_message = " and branch";
+                if !branch.is_valid() {
+                    println!("Branch: {}", branch.name);
+                    for violation in &branch.violations {
+                        violation_count += 1;
+                        println!("  {}: {}", violation.rule, violation.message);
+                    }
+                }
+            }
+            Err(error) => branch_error = Some(error),
         }
     }
 
     if violation_count > 0 {
         println!();
     }
-    let plural = if commit_count != 1 { "s" } else { "" };
-    println!(
-        "{} commit{} inspected, {} violations detected",
-        commit_count, plural, violation_count
+    let commit_plural = if commit_count != 1 { "s" } else { "" };
+    let violation_plural = if violation_count != 1 { "s" } else { "" };
+    print!(
+        "{} commit{}{} inspected",
+        commit_count, commit_plural, branch_message
     );
+    print!(
+        ", {} violation{} detected",
+        violation_count, violation_plural
+    );
+    if ignored_commit_count > 0 || debug {
+        let ignored_plural = if ignored_commit_count != 1 { "s" } else { "" };
+        print!(
+            " ({} commit{} ignored)",
+            ignored_commit_count, ignored_plural
+        );
+    }
+    println!();
+    let mut has_error = false;
+    if commit_result.is_err() {
+        has_error = true;
+        error!("An error occurred validating commits: {:?}", commit_result);
+    }
+    if branch_error.is_some() {
+        has_error = true;
+        error!(
+            "An error occurred validating the branch: {:?}",
+            &branch_error
+        );
+    }
+    if has_error {
+        std::process::exit(2)
+    }
     if violation_count > 0 {
         std::process::exit(1)
     }
@@ -213,6 +277,27 @@ mod tests {
         }
     }
 
+    fn checkout_branch(dir: &PathBuf, name: &str) {
+        let output = Command::new("git")
+            .args(&["checkout", "-b", name])
+            .current_dir(&dir)
+            .stdin(Stdio::null())
+            .output()
+            .expect(&format!("Could not checkout branch: {}", name));
+        if !output.status.success() {
+            panic!(
+                "Failed to checkout branch: {}\nExit code: {}\nSDTOUT: {}\nSTDERR: {}",
+                name,
+                output
+                    .status
+                    .code()
+                    .expect("Could not fetch status code of git checkout"),
+                String::from_utf8(output.stdout).unwrap(),
+                String::from_utf8(output.stderr).unwrap()
+            )
+        }
+    }
+
     fn create_commit(dir: &PathBuf, subject: &str, message: &str) {
         let mut args = vec![
             "commit".to_string(),
@@ -313,7 +398,7 @@ mod tests {
                 "{}: Test commit",
                 short_sha
             )))
-            .stdout(predicate::str::contains("1 commit inspected"));
+            .stdout(predicate::str::contains("1 commit and branch inspected"));
     }
 
     #[test]
@@ -328,7 +413,7 @@ mod tests {
         let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
         let assert = cmd.current_dir(dir).assert().success();
         assert.stdout(predicate::str::contains(
-            "1 commit inspected, 0 violations detected\n",
+            "1 commit and branch inspected, 0 violations detected\n",
         ));
     }
 
@@ -344,13 +429,73 @@ mod tests {
             .stdout(predicate::str::contains(
                 "Fixing tests\n\
                 \x20\x20SubjectMood: Use the imperative mood for the commit subject.\n\
+                \x20\x20SubjectCliche: Reword the subject to describe the change in more detail.\n\
                 \x20\x20MessagePresence: Add a message body to provide more context about the change and why it was made.",
             ))
             .stdout(predicate::str::contains(
-                "1 commit inspected, 2 violations detected\n",
+                "1 commit and branch inspected, 3 violations detected\n",
             ));
     }
 
+    #[test]
+    fn test_single_commit_invalid_one_violation() {
+        compile_bin();
+        let dir = test_dir("single_commit_invalid_one_violation");
+        create_test_repo(
+            &dir,
+            &[("added some code", ""), ("Valid commit subject", "")],
+        );
+
+        let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
+        let assert = cmd.current_dir(dir).assert().failure().code(1);
+        assert
+            .stdout(predicate::str::contains(
+                "Valid commit subject\n\
+                \x20\x20MessagePresence: Add a message body to provide more context about the change and why it was made.",
+            ))
+            .stdout(predicate::str::contains(
+                "1 commit and branch inspected, 1 violation detected\n",
+            ));
+    }
+
+    #[test]
+    fn test_single_commit_ignored() {
+        compile_bin();
+        let dir = test_dir("single_commit_ignored");
+        create_test_repo(
+            &dir,
+            &[
+                ("added some code", ""),
+                ("Merge pull request #123 from tombruijn/repo", ""),
+            ],
+        );
+
+        let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
+        let assert = cmd.current_dir(dir).assert().success();
+        assert.stdout(predicate::str::contains(
+            "0 commits and branch inspected, 0 violations detected (1 commit ignored)\n",
+        ));
+    }
+
+    #[test]
+    fn test_single_commit_with_debug() {
+        compile_bin();
+        let dir = test_dir("single_commit_with_debug");
+        create_test_repo(
+            &dir,
+            &[
+                ("added some code", ""),
+                ("Valid commit subject", "Valid message body"),
+            ],
+        );
+
+        let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
+        let assert = cmd.current_dir(dir).args(["--debug"]).assert().success();
+        assert.stdout(predicate::str::contains(
+            "1 commit and branch inspected, 0 violations detected (0 commits ignored)\n",
+        ));
+    }
+
     #[test]
     fn test_multiple_commit_invalid() {
         compile_bin();
@@ -374,15 +519,16 @@ mod tests {
             .stdout(predicate::str::contains(
                 "added some code\n\
                 \x20\x20SubjectMood: Use the imperative mood for the commit subject.\n\
-                \x20\x20SubjectCapitalization: Start the commit subject a capital letter.",
+                \x20\x20SubjectCapitalization: Start the commit subject with a capital letter.",
             ))
             .stdout(predicate::str::contains(
                 "Fixing tests\n\
                 \x20\x20SubjectMood: Use the imperative mood for the commit subject.\n\
+                \x20\x20SubjectCliche: Reword the subject to describe the change in more detail.\n\
                 \x20\x20MessagePresence: Add a message body to provide more context about the change and why it was made.",
             ))
             .stdout(predicate::str::contains(
-                "2 commits inspected, 4 violations detected\n",
+                "2 commits and branch inspected, 5 violations detected\n",
             ));
     }
 
@@ -407,7 +553,7 @@ mod tests {
         assert.stdout(predicate::str::contains(
             "added some code\n\
              \x20\x20SubjectMood: Use the imperative mood for the commit subject.\n\
-             \x20\x20SubjectCapitalization: Start the commit subject a capital letter.",
+             \x20\x20SubjectCapitalization: Start the commit subject with a capital letter.",
         ));
     }
 
@@ -484,4 +630,61 @@ mod tests {
             "Unable to open commit message file: commit_message_file",
         ));
     }
+
+    #[test]
+    fn test_branch_valid() {
+        compile_bin();
+        let dir = test_dir("branch_valid");
+        create_test_repo(
+            &dir,
+            &[("Test commit", "I am a test commit, short but valid.")],
+        );
+        checkout_branch(&dir, "my-branch");
+
+        let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
+        let assert = cmd.current_dir(dir).assert().success();
+        assert.stdout(predicate::str::contains(
+            "1 commit and branch inspected, 0 violations detected\n",
+        ));
+    }
+
+    #[test]
+    fn test_branch_invalid() {
+        compile_bin();
+        let dir = test_dir("branch_invalid");
+        create_test_repo(
+            &dir,
+            &[("Test commit", "I am a test commit, short but valid.")],
+        );
+        checkout_branch(&dir, "fix-123");
+
+        let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
+        let assert = cmd.current_dir(dir).assert().failure().code(1);
+        assert
+            .stdout(predicate::str::contains(
+                "Branch: fix-123\n\
+                \x20\x20BranchNameTicketNumber: Remove the ticket number from the branch name or expand the branch name with more details.\n\
+                \x20\x20BranchNameCliche: Reword the branch name to describe the change in more detail.",
+            ))
+            .stdout(predicate::str::contains(
+                    "1 commit and branch inspected, 2 violations detected\n",
+            ));
+    }
+
+    #[test]
+    fn test_no_branch_validation() {
+        compile_bin();
+        let dir = test_dir("branch_invalid_disabled");
+        create_test_repo(
+            &dir,
+            &[("Test commit", "I am a test commit, short but valid.")],
+        );
+        checkout_branch(&dir, "fix-123");
+
+        let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
+        let assert = cmd.arg("--no-branch").current_dir(dir).assert().success();
+        assert.stdout(predicate::str::contains(
+            "1 commit inspected, 0 violations detected\n",
+        ));
+    }
 }
