@@ -16,6 +16,10 @@ pub enum Rule {
     MessageEmptyFirstLine,
     MessagePresence,
     MessageLineLength,
+    BranchNameTicketNumber,
+    BranchNameLength,
+    BranchNamePunctuation,
+    BranchNameCliche,
 }
 
 #[derive(Debug)]
@@ -41,6 +45,10 @@ impl fmt::Display for Rule {
             Rule::MessageEmptyFirstLine => "MessageEmptyFirstLine",
             Rule::MessagePresence => "MessagePresence",
             Rule::MessageLineLength => "MessageLineLength",
+            Rule::BranchNameTicketNumber => "BranchNameTicketNumber",
+            Rule::BranchNameLength => "BranchNameLength",
+            Rule::BranchNamePunctuation => "BranchNamePunctuation",
+            Rule::BranchNameCliche => "BranchNameCliche",
         };
         write!(f, "{}", label)
     }
